@@ -10,10 +10,11 @@ import decimal
 import base64
 import copy
 import functools
+import datetime
 from collections import defaultdict
 
 
-__version__ = '1.20.1'
+__version__ = '1.21.0'
 _PARAMS = re.compile(r'{\w+}')
 
 # Implementation note:  This file is intended to be a standalone file
@@ -695,6 +696,26 @@ class DecoratorAPI(object):
                                  'description': description},
         )
 
+    def on_kinesis_record(self, stream, batch_size=100,
+                          starting_position='LATEST', name=None):
+        return self._create_registration_function(
+            handler_type='on_kinesis_record',
+            name=name,
+            registration_kwargs={'stream': stream,
+                                 'batch_size': batch_size,
+                                 'starting_position': starting_position},
+        )
+
+    def on_dynamodb_record(self, stream_arn, batch_size=100,
+                           starting_position='LATEST', name=None):
+        return self._create_registration_function(
+            handler_type='on_dynamodb_record',
+            name=name,
+            registration_kwargs={'stream_arn': stream_arn,
+                                 'batch_size': batch_size,
+                                 'starting_position': starting_position},
+        )
+
     def route(self, path, **kwargs):
         return self._create_registration_function(
             handler_type='route',
@@ -748,23 +769,7 @@ class DecoratorAPI(object):
         return _register_handler
 
     def _wrap_handler(self, handler_type, handler_name, user_handler):
-        event_classes = {
-            'on_s3_event': S3Event,
-            'on_sns_message': SNSEvent,
-            'on_sqs_message': SQSEvent,
-            'on_cw_event': CloudWatchEvent,
-            'schedule': CloudWatchEvent,
-            'lambda_function': LambdaFunctionEvent,
-        }
-        middleware_mapping = {
-            'on_s3_event': 's3',
-            'on_sns_message': 'sns',
-            'on_sqs_message': 'sqs',
-            'on_cw_event': 'cloudwatch',
-            'schedule': 'scheduled',
-            'lambda_function': 'pure_lambda',
-        }
-        if handler_type in event_classes:
+        if handler_type in _EVENT_CLASSES:
             if handler_type == 'lambda_function':
                 # We have to wrap existing @app.lambda_function()
                 # handlers for backwards compat reasons so we can
@@ -774,9 +779,9 @@ class DecoratorAPI(object):
                 # here.
                 user_handler = PureLambdaWrapper(user_handler)
             return EventSourceHandler(
-                user_handler, event_classes[handler_type],
+                user_handler, _EVENT_CLASSES[handler_type],
                 middleware_handlers=self._get_middleware_handlers(
-                    event_type=middleware_mapping[handler_type],
+                    event_type=_MIDDLEWARE_MAPPING[handler_type],
                 )
             )
 
@@ -933,6 +938,28 @@ class _HandlerRegistration(object):
         )
         self.event_sources.append(sqs_config)
 
+    def _register_on_kinesis_record(self, name, handler_string,
+                                    kwargs, **unused):
+        kinesis_config = KinesisEventConfig(
+            name=name,
+            handler_string=handler_string,
+            stream=kwargs['stream'],
+            batch_size=kwargs['batch_size'],
+            starting_position=kwargs['starting_position'],
+        )
+        self.event_sources.append(kinesis_config)
+
+    def _register_on_dynamodb_record(self, name, handler_string,
+                                     kwargs, **unused):
+        ddb_config = DynamoDBEventConfig(
+            name=name,
+            handler_string=handler_string,
+            stream_arn=kwargs['stream_arn'],
+            batch_size=kwargs['batch_size'],
+            starting_position=kwargs['starting_position'],
+        )
+        self.event_sources.append(ddb_config)
+
     def _register_on_cw_event(self, name, handler_string, kwargs, **unused):
         event_source = CloudWatchEventConfig(
             name=name,
@@ -1374,6 +1401,24 @@ class SQSEventConfig(BaseEventSourceConfig):
         self.batch_size = batch_size
 
 
+class KinesisEventConfig(BaseEventSourceConfig):
+    def __init__(self, name, handler_string, stream,
+                 batch_size, starting_position):
+        super(KinesisEventConfig, self).__init__(name, handler_string)
+        self.stream = stream
+        self.batch_size = batch_size
+        self.starting_position = starting_position
+
+
+class DynamoDBEventConfig(BaseEventSourceConfig):
+    def __init__(self, name, handler_string, stream_arn,
+                 batch_size, starting_position):
+        super(DynamoDBEventConfig, self).__init__(name, handler_string)
+        self.stream_arn = stream_arn
+        self.batch_size = batch_size
+        self.starting_position = starting_position
+
+
 class WebsocketConnectConfig(BaseEventSourceConfig):
     CONNECT_ROUTE = '$connect'
 
@@ -1441,6 +1486,14 @@ class EventSourceHandler(BaseLambdaHandler):
         self._middleware_handlers = middleware_handlers
         self.handler = None
 
+    @property
+    def middleware_handlers(self):
+        return self._middleware_handlers
+
+    @middleware_handlers.setter
+    def middleware_handlers(self, value):
+        self._middleware_handlers = value
+
     def __call__(self, event, context):
         event_obj = self.event_class(event, context)
         if self.handler is None:
@@ -1726,6 +1779,71 @@ class SQSRecord(BaseLambdaEvent):
         self.receipt_handle = event_dict['receiptHandle']
 
 
+class KinesisEvent(BaseLambdaEvent):
+    def _extract_attributes(self, event_dict):
+        pass
+
+    def __iter__(self):
+        for record in self._event_dict['Records']:
+            yield KinesisRecord(record, self.context)
+
+
+class KinesisRecord(BaseLambdaEvent):
+    def _extract_attributes(self, event_dict):
+        kinesis = event_dict['kinesis']
+        encoded_payload = kinesis['data']
+        self.data = base64.b64decode(encoded_payload)
+        self.sequence_number = kinesis['sequenceNumber']
+        self.partition_key = kinesis['partitionKey']
+        self.schema_version = kinesis['kinesisSchemaVersion']
+        self.timestamp = datetime.datetime.utcfromtimestamp(
+            kinesis['approximateArrivalTimestamp'])
+
+
+class DynamoDBEvent(BaseLambdaEvent):
+    def _extract_attributes(self, event_dict):
+        pass
+
+    def __iter__(self):
+        for record in self._event_dict['Records']:
+            yield DynamoDBRecord(record, self.context)
+
+
+class DynamoDBRecord(BaseLambdaEvent):
+    def _extract_attributes(self, event_dict):
+        dynamodb = event_dict['dynamodb']
+        self.timestamp = datetime.datetime.utcfromtimestamp(
+            dynamodb['ApproximateCreationDateTime'])
+        self.keys = dynamodb.get('Keys')
+        self.new_image = dynamodb.get('NewImage')
+        self.old_image = dynamodb.get('OldImage')
+        self.sequence_number = dynamodb['SequenceNumber']
+        self.size_bytes = dynamodb['SizeBytes']
+        self.stream_view_type = dynamodb['StreamViewType']
+        # These are from the top level keys in a record.
+        self.aws_region = event_dict['awsRegion']
+        self.event_id = event_dict['eventID']
+        self.event_name = event_dict['eventName']
+        self.event_source_arn = event_dict['eventSourceARN']
+
+    @property
+    def table_name(self):
+        # Converts:
+        # "arn:aws:dynamodb:us-west-2:12345:table/MyTable/"
+        # "stream/2020-09-28T16:49:14.209"
+        #
+        # into:
+        # "MyTable"
+        parts = self.event_source_arn.split(':', 5)
+        if not len(parts) == 6:
+            return ''
+        full_name = parts[-1]
+        name_parts = full_name.split('/')
+        if len(name_parts) >= 2:
+            return name_parts[1]
+        return ''
+
+
 class Blueprint(DecoratorAPI):
     def __init__(self, import_name):
         self._import_name = import_name
@@ -1776,17 +1894,6 @@ class Blueprint(DecoratorAPI):
     # these methods are not public in the sense that we don't want users to
     # call them, they're available for blueprints to use in order to avoid
     # boilerplate code.
-    def _wrap_handler(self, handler_type, handler_name, user_handler):
-        # This deferred wrap handler will be called when we call
-        # Blueprint.register() and provide an app object.  Once
-        # we have access to an app, we can call the actual _wrap_handler
-        # which will have access to middleware.
-        @functools.wraps(user_handler)
-        def _defer_wrap_handler(app):
-            # pylint: disable=protected-access
-            return app._wrap_handler(handler_type, handler_name,
-                                     user_handler)
-        return _defer_wrap_handler
 
     def register_middleware(self, func, event_type='all'):
         self._deferred_registrations.append(
@@ -1802,18 +1909,22 @@ class Blueprint(DecoratorAPI):
         # we have to duplicate either the methods or the params in this
         # class.  We're using _register_handler as a tradeoff for cutting
         # down on the duplication.
-        self._deferred_registrations.append(
+        def _register_blueprint_handler(app, options):
+            if handler_type in _EVENT_CLASSES:
+                # pylint: disable=protected-access
+                wrapped_handler.middleware_handlers = \
+                    app._get_middleware_handlers(
+                        _MIDDLEWARE_MAPPING[handler_type])
             # pylint: disable=protected-access
-            lambda app, options: app._register_handler(
-                handler_type, name, user_handler, wrapped_handler(app),
+            app._register_handler(
+                handler_type, name, user_handler, wrapped_handler,
                 kwargs, options
             )
-        )
+        self._deferred_registrations.append(_register_blueprint_handler)
 
     def _get_middleware_handlers(self, event_type):
-        raise RuntimeError("Blueprints are unable to retrieve middleware "
-                           "handlers, must be registered to an app using "
-                           "register_blueprint()")
+        # This will get filled in later during the registration process.
+        return []
 
 
 # This class is used to convert any existing/3rd party decorators
@@ -1847,3 +1958,27 @@ class ConvertToMiddleware(object):
         if isinstance(event, Request):
             return event.to_original_event(), event.lambda_context
         return event.to_dict(), event.context
+
+
+_EVENT_CLASSES = {
+    'on_s3_event': S3Event,
+    'on_sns_message': SNSEvent,
+    'on_sqs_message': SQSEvent,
+    'on_cw_event': CloudWatchEvent,
+    'on_kinesis_record': KinesisEvent,
+    'on_dynamodb_record': DynamoDBEvent,
+    'schedule': CloudWatchEvent,
+    'lambda_function': LambdaFunctionEvent,
+}
+
+
+_MIDDLEWARE_MAPPING = {
+    'on_s3_event': 's3',
+    'on_sns_message': 'sns',
+    'on_sqs_message': 'sqs',
+    'on_cw_event': 'cloudwatch',
+    'on_kinesis_record': 'kinesis',
+    'on_dynamodb_record': 'dynamodb',
+    'schedule': 'scheduled',
+    'lambda_function': 'pure_lambda',
+}
