@@ -363,12 +363,14 @@ class TypedAWSClient(object):
                         handler,                     # type: str
                         environment_variables=None,  # type: StrMap
                         tags=None,                   # type: StrMap
+                        xray=None,                   # type: Optional[bool]
                         timeout=None,                # type: OptInt
                         memory_size=None,            # type: OptInt
                         security_group_ids=None,     # type: OptStrList
                         subnet_ids=None,             # type: OptStrList
                         layers=None,                 # type: OptStrList
                         ):
+        # pylint: disable=too-many-locals
         # type: (...) -> str
         kwargs = {
             'FunctionName': function_name,
@@ -381,6 +383,8 @@ class TypedAWSClient(object):
             kwargs['Environment'] = {"Variables": environment_variables}
         if tags is not None:
             kwargs['Tags'] = tags
+        if xray is True:
+            kwargs['TracingConfig'] = {'Mode': 'Active'}
         if timeout is not None:
             kwargs['Timeout'] = timeout
         if memory_size is not None:
@@ -834,6 +838,7 @@ class TypedAWSClient(object):
                         environment_variables=None,  # type: StrMap
                         runtime=None,                # type: OptStr
                         tags=None,                   # type: StrMap
+                        xray=None,                   # type: bool
                         timeout=None,                # type: OptInt
                         memory_size=None,            # type: OptInt
                         role_arn=None,               # type: OptStr
@@ -856,6 +861,7 @@ class TypedAWSClient(object):
             timeout=timeout,
             memory_size=memory_size,
             role_arn=role_arn,
+            xray=xray,
             subnet_ids=subnet_ids,
             security_group_ids=security_group_ids,
             function_name=function_name,
@@ -903,6 +909,7 @@ class TypedAWSClient(object):
                                 security_group_ids,  # type: OptStrList
                                 function_name,  # type: str
                                 layers,  # type: OptStrList
+                                xray,  # type: Optional[bool]
                                 ):
         # type: (...) -> None
         kwargs = {}  # type: Dict[str, Any]
@@ -916,6 +923,8 @@ class TypedAWSClient(object):
             kwargs['MemorySize'] = memory_size
         if role_arn is not None:
             kwargs['Role'] = role_arn
+        if xray:
+            kwargs['TracingConfig'] = {'Mode': 'Active'}
         if security_group_ids is not None and subnet_ids is not None:
             kwargs['VpcConfig'] = self._create_vpc_config(
                 subnet_ids=subnet_ids,
@@ -1074,12 +1083,13 @@ class TypedAWSClient(object):
         except client.exceptions.NotFoundException:
             raise ResourceDoesNotExistError(rest_api_id)
 
-    def deploy_rest_api(self, rest_api_id, api_gateway_stage):
-        # type: (str, str) -> None
+    def deploy_rest_api(self, rest_api_id, api_gateway_stage, xray):
+        # type: (str, str, bool) -> None
         client = self._client('apigateway')
         client.create_deployment(
             restApiId=rest_api_id,
             stageName=api_gateway_stage,
+            tracingEnabled=bool(xray),
         )
 
     def add_permission_for_apigateway(self, function_name,
@@ -1591,20 +1601,23 @@ class TypedAWSClient(object):
                     StatementId=statement['Sid'],
                 )
 
-    def create_sqs_event_source(self, queue_arn, function_name, batch_size):
-        # type: (str, str, int) -> None
+    def create_lambda_event_source(self, event_source_arn, function_name,
+                                   batch_size, starting_position=None):
+        # type: (str, str, int, Optional[str]) -> None
         lambda_client = self._client('lambda')
         kwargs = {
-            'EventSourceArn': queue_arn,
+            'EventSourceArn': event_source_arn,
             'FunctionName': function_name,
             'BatchSize': batch_size,
         }
+        if starting_position is not None:
+            kwargs['StartingPosition'] = starting_position
         return self._call_client_method_with_retries(
             lambda_client.create_event_source_mapping, kwargs,
             max_attempts=self.LAMBDA_CREATE_ATTEMPTS
         )['UUID']
 
-    def update_sqs_event_source(self, event_uuid, batch_size):
+    def update_lambda_event_source(self, event_uuid, batch_size):
         # type: (str, int) -> None
         lambda_client = self._client('lambda')
         self._call_client_method_with_retries(
@@ -1614,7 +1627,7 @@ class TypedAWSClient(object):
             should_retry=self._is_settling_error,
         )
 
-    def remove_sqs_event_source(self, event_uuid):
+    def remove_lambda_event_source(self, event_uuid):
         # type: (str) -> None
         lambda_client = self._client('lambda')
         self._call_client_method_with_retries(
@@ -1652,6 +1665,27 @@ class TypedAWSClient(object):
         except client.exceptions.ResourceNotFoundException:
             return False
 
+    def verify_event_source_arn_current(self, event_uuid, event_source_arn,
+                                        function_arn):
+        # type: (str, str, str) -> bool
+        """Check if the uuid matches the event and function ARN.
+
+        This is similar to verify_event_source_current, except that you provide
+        an explicit event_source_arn here.  This is useful for cases where you
+        know the event source ARN or where you can't construct the event source
+        arn solely based on the resource_name and the service_name.
+
+        """
+        client = self._client('lambda')
+        try:
+            attributes = client.get_event_source_mapping(UUID=event_uuid)
+        except client.exceptions.ResourceNotFoundException:
+            return False
+        return bool(
+            event_source_arn == attributes['EventSourceArn'] and
+            function_arn == attributes['FunctionArn']
+        )
+
     def create_websocket_api(self, name):
         # type: (str) -> str
         client = self._client('apigatewayv2')
